@@ -78,6 +78,22 @@ public class CodeGenBugTest extends ProcessorTestBase {
         env = null;
     }
 
+    @Test
+    // Test for CXF-1678
+    public void testLogicalOnlyWSDL() throws Exception {
+        env.put(ToolConstants.CFG_WSDLURL, 
+                getLocation("/wsdl2java_wsdl/cxf-1678/hello_world_logical_only.wsdl"));
+        processor.setContext(env);
+        processor.execute();
+
+        assertNotNull("Trouble processing logical only wsdl", output);
+
+        Class clz = classLoader.loadClass("org.apache.cxf.cxf1678.hello_world_soap_http.GreeterImpl");
+        WebService webServiceAnn = AnnotationUtil.getPrivClassAnnotation(clz, WebService.class);
+        assertEquals("org.apache.cxf.cxf1678.hello_world_soap_http.Greeter",
+                     webServiceAnn.endpointInterface());
+    }
+
     @Test
     public void testBug305729() throws Exception {
         env.put(ToolConstants.CFG_WSDLURL, getLocation("/wsdl2java_wsdl/bug305729/hello_world.wsdl"));
@@ -100,7 +116,8 @@ public class CodeGenBugTest extends ProcessorTestBase {
         Class clz = classLoader.loadClass("org.apache.hello_world_soap_http.GreeterImpl");
 
         WebService webServiceAnn = AnnotationUtil.getPrivClassAnnotation(clz, WebService.class);
-        assertEquals("Greeter", webServiceAnn.name());
+        assertTrue("Impl class should note generate name property value in webService annotation",
+                    webServiceAnn.name().equals(""));
         assertFalse("Impl class should generate portName property value in webService annotation",
                     webServiceAnn.portName().equals(""));
         assertFalse("Impl class should generate serviceName property value in webService annotation",
@@ -450,6 +467,7 @@ public class CodeGenBugTest extends ProcessorTestBase {
     @Test
     public void testHelloWorldExternalBindingFile() throws Exception {
         Server server = new Server(8585);
+
         ResourceHandler reshandler = new ResourceHandler();
         reshandler.setResourceBase(getLocation("/wsdl2java_wsdl/"));
         server.addHandler(reshandler);
@@ -458,7 +476,12 @@ public class CodeGenBugTest extends ProcessorTestBase {
         env.put(ToolConstants.CFG_BINDING, "http://localhost:8585/remote-hello_world_binding.xsd");
         processor.setContext(env);
         processor.execute();
-        server.stop();
+        try {
+            reshandler.stop();
+        } finally {
+            server.stop();
+            server.destroy();
+        }
 
     }
 
@@ -500,14 +523,11 @@ public class CodeGenBugTest extends ProcessorTestBase {
                                       getLocation("/wsdl2java_wsdl/hello-mime.wsdl")};
 
         WSDLToJava.main(args);
+
         assertFileEquals(getClass().getResource("expected/expected_hello_mime").getFile(),
                          output.getCanonicalPath() + "/org/apache/hello_world_mime/Hello.java");
-
-
-
     }
 
-
     @Test
     public void testWebResult() throws Exception {
 
@@ -515,9 +535,13 @@ public class CodeGenBugTest extends ProcessorTestBase {
         processor.setContext(env);
         processor.execute();
 
-        assertFileEquals(getClass().getResource("expected/expected_sayHi").getFile(),
-                         output.getCanonicalPath() + "/org/apache/sayhi/SayHi.java");
-
+        String results = getStringFromFile(new File(output.getCanonicalPath(), 
+                                                    "org/apache/sayhi/SayHi.java"));
+        assertTrue(results.trim().length() > 0);
+        
+        assertTrue(results.indexOf("@WebResult(name  =  \"return\",  " 
+                                   + "targetNamespace  =  \"http://apache.org/sayHi\")") != -1);
+        assertTrue(results.indexOf("@WebResult(name  =  \"return\",  targetNamespace  =  \"\")") != -1);
     }
 
 
@@ -527,7 +551,11 @@ public class CodeGenBugTest extends ProcessorTestBase {
         env.put(ToolConstants.CFG_WSDLURL, getLocation("/wsdl2java_wsdl/bug627/hello_world.wsdl"));
         env.put(ToolConstants.CFG_BINDING, getLocation("/wsdl2java_wsdl/bug627/async_binding.xml"));
         processor.setContext(env);
-        processor.execute();
+        try {
+            processor.execute();
+        } catch (Exception ex) {
+            //ignore
+        }
 
 
         Class clz = classLoader.loadClass("org.apache.hello_world_soap_http.Greeter");
@@ -644,8 +672,12 @@ public class CodeGenBugTest extends ProcessorTestBase {
             String ns = "http://bugs.cxf/services/bug1";
             QName bug1 = new QName(ns, "myBug1");
             QName bug2 = new QName(ns, "myBug2");
-            Message msg = new Message("NON_UNIQUE_BODY", UniqueBodyValidator.LOG, bug1, bug1, bug2, bug1);
-            assertEquals(msg.toString().trim(), e.getMessage().trim());
+            Message msg1 = new Message("NON_UNIQUE_BODY", UniqueBodyValidator.LOG, bug1, bug1, bug2, bug1);
+            Message msg2 = new Message("NON_UNIQUE_BODY", UniqueBodyValidator.LOG, bug1, bug2, bug1, bug1);
+
+            boolean boolA = msg1.toString().trim().equals(e.getMessage().trim());
+            boolean boolB = msg2.toString().trim().equals(e.getMessage().trim());
+            assertTrue(boolA || boolB);
         }
     }
 
@@ -668,6 +700,13 @@ public class CodeGenBugTest extends ProcessorTestBase {
             assertEquals(msg.toString().trim(), e.getMessage().trim());
         }
     }
+    @Test
+    public void testNonWrapperStyleNameCollision() throws Exception {
+        env.put(ToolConstants.CFG_WSDLURL,
+                getLocation("/wsdl2java_wsdl/cxf918/bug2.wsdl"));
+        processor.setContext(env);
+        processor.execute();
+    }
 
     @Test
     public void testParameterOrderNoOutputMessage() throws Exception {
@@ -692,6 +731,8 @@ public class CodeGenBugTest extends ProcessorTestBase {
             String results = getStringFromFile(new File(output.getCanonicalPath(), 
                                                         "org/tempuri/GreeterRPCLit.java"));
             assertTrue(results.indexOf("@WebParam(partName  =  \"inInt\",  name  =  \"inInt\")") != -1);
+            assertTrue(results.indexOf("Style.RPC") != -1);
+            
         } catch (Exception e) {
             fail("The cxf978.wsdl is a valid wsdl, should pass the test, caused by: " + e.getMessage());
         }
@@ -725,7 +766,8 @@ public class CodeGenBugTest extends ProcessorTestBase {
         String results = getStringFromFile(new File(output.getCanonicalPath(), 
                                                     "soapinterface/ems/esendex/com/AccountServiceSoap.java"));
         assertTrue(results.indexOf("public  int  getMessageLimit") != -1);
-        assertTrue(results.indexOf("header  =  true,  name  =  \"MessengerHeader") != -1);
+        assertTrue(results.indexOf("name  =  \"MessengerHeader") != -1);
+        assertTrue(results.indexOf("header  =  true") != -1);
     }
     
     @Test
@@ -903,4 +945,53 @@ public class CodeGenBugTest extends ProcessorTestBase {
         assertEquals("int", webFault.name());
     }
     
+    @Test
+    public void testCXF1620() throws Exception {
+        env.put(ToolConstants.CFG_WSDLURL, 
+                getLocation("/wsdl2java_wsdl/jaxb_custom_extensors.wsdl"));   
+        env.put(ToolConstants.CFG_BINDING,
+                getLocation("/wsdl2java_wsdl/jaxb_custom_extensors.xjb"));
+
+        processor.setContext(env);
+        processor.execute();
+        Class<?> clz = classLoader.loadClass("org.apache.cxf.w2j.jaxb_custom_ext.types.Foo");
+        
+        assertEquals(3, clz.getDeclaredFields().length);
+        
+        clz = classLoader.loadClass("org.apache.cxf.w2j.jaxb_custom_ext.types.Foo2");
+        assertEquals(1, clz.getDeclaredFields().length);
+    }
+ 
+    
+    @Test
+    public void testCXF1048() throws Exception {
+
+        env.put(ToolConstants.CFG_COMPILE, "compile");
+        env.put(ToolConstants.CFG_IMPL, ToolConstants.CFG_IMPL);
+        env.put(ToolConstants.CFG_OUTPUTDIR, output.getCanonicalPath());
+        env.put(ToolConstants.CFG_CLASSDIR, output.getCanonicalPath() + "/classes");
+        env.put(ToolConstants.CFG_WSDLURL, getLocation("/wsdl2java_wsdl/cxf1048/test.wsdl"));
+        processor.setContext(env);
+        processor.execute();
+        Class clz = classLoader.loadClass("org.apache.hello_world_soap_http.PingImpl");
+
+        WebService webServiceAnn = AnnotationUtil.getPrivClassAnnotation(clz, WebService.class);
+        assertEquals("org.apache.hello_world_soap_http.Ping", webServiceAnn.endpointInterface());
+        assertEquals("GreeterSOAPService", webServiceAnn.serviceName());
+        assertEquals("PingSoapPort", webServiceAnn.portName());
+    }
+
+    
+    @Test
+    public void testCXF1694() throws Exception {
+
+        try {
+            env.put(ToolConstants.CFG_WSDLURL, getLocation("/wsdl2java_wsdl/cxf1694/test.wsdl"));
+            processor.setContext(env);
+            processor.execute();
+        } catch (Exception ex) {
+            assertTrue(ex.getMessage().contains("{http://child/}Binding is not correct"));
+        }
+    }
+    
 }
